 /**************************************************************************
 *   Copyright (C) 2005 by Vlad Svetlichny                                 *
 *   vladsv@inbox.ru                                                       *
 *   Distributed under GPL                                                 *
 ***************************************************************************/
 #include "statisticsb.h"
 #include <qlayout.h>
 #include <qpushbutton.h>
 #include <qtabwidget.h>   
 #include <qpixmap.h>
 #include <qregexp.h>
 #include <qstringlist.h>
 #include "tab.h"
 #include "kniga.h"


StatisticSB::StatisticSB(QWidget *parent, Kniga *kkniga)
 : QWidget(parent)
{
    fkniga=kkniga;
    QGridLayout *grid=new QGridLayout(this,8,2,0,5);
    grid->addWidget(new QLabel(tr("Paragraphs:"),this),0,0);
    grid->addWidget(new QLabel(tr("Lines:"),this),1,0);
//     grid->addWidget(new QLabel(tr("Sentences:"),this),2,0);
//     grid->addWidget(new QLabel(tr("Words:"),this),3,0);
    grid->addWidget(new QLabel(tr("Characters:"),this),2,0);
    grid->addWidget(new QLabel(tr("Without spaces:"),this),3,0);
    grid->addWidget(new QLabel(tr("Words:"),this),4,0);
    grid->addWidget(new QLabel(tr("Syllabels:"),this),5,0);
    grid->addWidget(new QLabel(tr("Sentences:"),this),6,0);
    QPushButton *btnRefresh=new QPushButton(
    QPixmap::fromMimeSource("reload.png"),tr("Refresh"),this);
    grid->addMultiCellWidget(btnRefresh,7,7,0,1,Qt::AlignBottom);
    grid->addWidget(lblPara=new QLabel(this),0,1);
    grid->addWidget(lblLines=new QLabel(this),1,1);
    grid->addWidget(lblWithSpaces=new QLabel(this),2,1);
    grid->addWidget(lblWithoutSpaces=new QLabel(this),3,1);
    grid->addWidget(lblWords=new QLabel(this),4,1);
    grid->addWidget(lblSyllabels=new QLabel(this),5,1);
    grid->addWidget(lblSentences=new QLabel(this),6,1);
    
    connect(btnRefresh,SIGNAL(clicked()),SLOT(refresh()));
}

void StatisticSB::refresh()
{
   QTabWidget *tw=(QTabWidget*)(fkniga->tw);
   if (tw->count())
   {
       tab *t=(tab*)(tw->currentPage());
       lblPara->setText(QString::number(t->paragraphs()));
       lblWithoutSpaces->setText(QString::number(
        t->length()-(t->text().contains(" "))));
       lblWithSpaces->setText(QString::number(t->length()));
       lblLines->setText(QString::number(t->lines()));
       
       QString s=t->text();
       
       int words=0;
       int syllables=0;
       int sentences=0;
       
       //completely from Kword - kwtextframetest.cc
       
       QStringList subs_syl;
       subs_syl << "cial" << "tia" << "cius" << "cious" <<"giu"<< "ion"<<"iou";
       QStringList subs_syl_regexp;
       subs_syl_regexp << "sia$" << "ely$";

       QStringList add_syl;
       add_syl << "ia" << "riet" << "dien" << "iu" << "io" << "ii";
       QStringList add_syl_regexp;
       add_syl_regexp << "[aeiouym]bl$" << "[aeiou]{3}" << "^mc" << "ism$"
       << "[^l]lien" << "^coa[dglx]." << "[^gq]ua[^auieo]" << "dnt$";
        
       QRegExp re("\\s+");
       QStringList wordlist = QStringList::split(re, s);
       words += wordlist.count();
       re.setCaseSensitive(false);
       for ( QStringList::Iterator it=wordlist.begin();it!=wordlist.end(); ++it ) {
            QString word = *it;
            re.setPattern("[!?.,:_\"-]");    // clean word from punctuation
            word.replace(re, "");
            if ( word.length() <= 3 ) {  // extension to the original algorithm
                syllables++;
                continue;
            }
            re.setPattern("e$");
            word.replace(re, "");
            re.setPattern("[^aeiouy]+");
            QStringList syls = QStringList::split(re, word);
            int word_syllables = 0;
            for ( QStringList::Iterator it = subs_syl.begin(); it != subs_syl.end(); ++it ) {
                if( word.find(*it, 0, false) != -1 )
                    word_syllables--;
            }
            for ( QStringList::Iterator it = subs_syl_regexp.begin(); it != subs_syl_regexp.end(); ++it ) {
                re.setPattern(*it);
                if( word.find(re) != -1 )
                    word_syllables--;
            }
            for ( QStringList::Iterator it = add_syl.begin(); it != add_syl.end(); ++it ) {
                if( word.find(*it, 0, false) != -1 )
                    word_syllables++;
            }
            for ( QStringList::Iterator it = add_syl_regexp.begin(); it != add_syl_regexp.end(); ++it ) {
                re.setPattern(*it);
                if( word.find(re) != -1 )
                    word_syllables++;
            }
            word_syllables += syls.count();
            if ( word_syllables == 0 )
                word_syllables = 1;
            syllables += word_syllables;
        }
        re.setCaseSensitive(true);
        
        //kwtextframetest.cc with some changes:)
        
        s = s.stripWhiteSpace();
        QChar lastchar = s.at(s.length());
        if( ! s.isEmpty() && !((lastchar=="!") || (lastchar=="?") || (lastchar=="."))) s = s + ".";
        re.setPattern("[.?!]+");         // count "..." as only one "."
        s.replace(re, ".");
        re.setPattern("\\d\\.\\d");      // don't count floating point numbers as sentences
        s.replace(re, "0,0");
        re.setPattern("[A-Z]\\.+");      // don't count "U.S.A." as three sentences
        s.replace(re, "*");
//         qDebug(s);
        for ( uint i = 0 ; i < s.length() ; ++i )
        {
            QChar ch = s[i];
//             qDebug(ch);
        switch(ch){
            case '.':
            case '!':
            case '?':
            ++sentences;
        }
        }

        
        lblWords->setText(QString::number(words));
        lblSyllabels->setText(QString::number(syllables));
        lblSentences->setText(QString::number(sentences));
   }
}
